/*
 Copyright (C) 2022 Ivan Polyakov

 This file is part of vilor's website.

 Vilor's website is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.

 Vilor's website is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */
#include <sys/types.h> 
#include <stdio.h> 
#include <fcgi_config.h>
#include <fcgiapp.h>
#include <string.h>
#include "server.h"
#include "routes.h"
#include "utils.h"

static int socketId; 

static int send_response(FCGX_Request *req, struct route *route)
{
    const char *resbuff;

    if (route->resh(route, req, &resbuff)) {
        fputs("Can't create response\n", stderr);
        return 2;
    }

    /* sending response */
    FCGX_PutS("Content-type: text/html" CRLF CRLF, req->out);
    FCGX_PutS(resbuff, req->out);
    FCGX_PutS(CRLF CRLF, req->out);

    return 0;
}

int main() 
{ 
    FCGX_Request req;

    FCGX_Init(); 

    if ((socketId = FCGX_OpenSocket(SOCKET_PATH, 20)) < 0) { 
        return 1; 
    } 

    if (FCGX_InitRequest(&req, socketId, 0)) { 
        fputs("Can't init request\n", stderr); 
        return 2; 
    }

    while (1) {
        int rc;
        struct route *route;

        rc = FCGX_Accept_r(&req);

        if (rc < 0) {
            fputs("Can't accept new request\n", stderr);
            break;
        }

        if (!(route = find_route(&req))) {
            fputs("Invalid URI\n", stderr);
            continue;
        }

        send_response(&req, route);

        FCGX_Finish_r(&req);
    }

    return 0; 
}
